function statusblock(ids)
	local checkthese = {}
	
	if (ids == nil) then
		for k,unit in ipairs(units) do
			table.insert(checkthese, unit)
		end
	else
		for i,v in ipairs(ids) do
			local vunit = mmf.newObject(v)
			table.insert(checkthese, vunit)
		end
	end
	
	for i,unit in pairs(checkthese) do
		unit.values[FLOAT] = 0
		local name = unit.strings[UNITNAME]
		
		if (unit.strings[UNITTYPE] == "text") then
			name = "text"
		end
		
		local isfloat = hasfeature(name,"is","float",unit.fixed)
		local right = hasfeature(name,"is","right",unit.fixed)
		local up = hasfeature(name,"is","up",unit.fixed)
		local left = hasfeature(name,"is","left",unit.fixed)
		local down = hasfeature(name,"is","down",unit.fixed)
		
		if (isfloat ~= nil) then
			unit.values[FLOAT] = 1
		end
		
		if (issleep(unit.fixed) == false) then
			if (right ~= nil) then
				updatedir(unit.fixed,0)
			end
			if (up ~= nil) then
				updatedir(unit.fixed,1)
			end
			if (left ~= nil) then
				updatedir(unit.fixed,2)
			end
			if (down ~= nil) then
				updatedir(unit.fixed,3)
			end
		end
	end
end

function moveblock()
	local isshift = findallfeature(nil,"is","shift")
	local isfollow = findfeature(nil,"follow",nil)
	
	local doned = {}
	
	if (isfollow ~= nil) then
		for h,j in ipairs(isfollow) do
			local allfollows = findall(j)
			
			if (#allfollows > 0) then
				for k,l in ipairs(allfollows) do
					if (issleep(l) == false) then
						local unit = mmf.newObject(l)
						local x,y,name = unit.values[XPOS],unit.values[YPOS],unit.strings[UNITNAME]
						local unitrules = {}
						
						if (unit.strings[UNITTYPE] == "text") then
							name = "text"
						end
						
						if (featureindex[name] ~= nil) then					
							for a,b in ipairs(featureindex[name]) do
								local baserule = b[1]
								local conds = b[2]
								
								local verb = baserule[2]
								
								if (verb == "follow") then
									if testcond(conds,l) then
										table.insert(unitrules, b)
									end
								end
							end
						end
						
						local follow = xthis(unitrules,name,"follow")
						
						if (#follow > 0) then
							local distance = 9999
							local targetdir = -1
							
							for i,v in ipairs(follow) do
								local these = findall({v})
								
								if (#these > 0) then
									for a,b in ipairs(these) do
										if (b ~= unit.fixed) then
											local funit = mmf.newObject(b)
											
											local fx,fy = funit.values[XPOS],funit.values[YPOS]
											
											local xdir = fx-x
											local ydir = fy-y
											local dist = math.abs(xdir) + math.abs(ydir)
											local fdir = -1
											
											if (math.abs(xdir) <= math.abs(ydir)) then
												if (ydir >= 0) then
													fdir = 3
												else
													fdir = 1
												end
											else
												if (xdir > 0) then
													fdir = 0
												else
													fdir = 2
												end
											end
											
											if (dist < distance) then
												distance = dist
												targetdir = fdir
											end
										end
									end
								end
							end
			
							if (targetdir >= 0) then
								updatedir(unit.fixed,targetdir)
							end
						end
					end
				end
			end
		end
	end
	
	doupdate()
	
	for a,unitid in ipairs(isshift) do
		local unit = mmf.newObject(unitid)
		local x,y,dir = unit.values[XPOS],unit.values[YPOS],unit.values[DIR]
		
		local things = findallhere(x,y,unitid)
		
		if (#things > 0) and (isgone(unitid) == false) then
			for e,f in ipairs(things) do
				if floating(unitid,f) then
					local newunit = mmf.newObject(f)
					local name = newunit.strings[UNITNAME] 
					
					newunit.values[DIR] = unit.values[DIR]
				end
			end
		end
	end
	
	doupdate()
end

function fallblock(things)
	local checks = {}
	
	if (things == nil) then
		local isfall = findallfeature(nil,"is","fall")

		for a,unitid in ipairs(isfall) do
			table.insert(checks, unitid)
		end
	else
		for a,unitid in ipairs(things) do
			table.insert(checks, unitid)
		end
	end
	
	local done = false
	
	while (done == false) do
		local settled = true
		
		if (#checks > 0) then
			for a,unitid in pairs(checks) do
				local unit = mmf.newObject(unitid)
				local x,y,dir = unit.values[XPOS],unit.values[YPOS],unit.values[DIR]
				local name = getname(unit)
				local onground = false
				
				while (onground == false) do
					local below,below_ = check(unitid,x,y,3,false,"fall")
					
					local result = 0
					for c,d in pairs(below) do
						if (d ~= 0) then
							result = 1
						end
					end
					
					if (result == 0) then
						update(unitid,x,y+1)
						y = y + 1
						block({unitid},true)
						settled = false
						
						local stillgoing = hasfeature(name,"is","fall",unitid,x,y)
						if (stillgoing == nil) then
							onground = true
							table.remove(checks, a)
						end
					else
						onground = true
					end
				end
			end
			
			if settled then
				done = true
			end
		else
			done = true
		end
	end
end

function block(ids,small_)
	local delthese = {}
	local doned = {}
	local unitsnow = #units
	local removalsound = 1
	local small = false
	
	if (small_ ~= nil) then
		small = small_
	end
	
	local checkthese = {}
	
	if (ids == nil) then
		for k,unit in ipairs(units) do
			table.insert(checkthese, unit)
		end
	else
		for i,v in ipairs(ids) do
			local vunit = mmf.newObject(v)
			table.insert(checkthese, vunit)
		end
	end
	
	for k,unit in ipairs(checkthese) do
		if (k <= unitsnow) then
			local unitrules = {}
			local unitid = unit.fixed
			local name = unit.strings[UNITNAME]
			local x,y,dir,moved,id = unit.values[XPOS],unit.values[YPOS],unit.values[DIR],unit.values[MOVED],unit.values[ID]
			local ox,oy = 0,0
			local online = true
			
			if (unit.strings[UNITTYPE] == "text") then
				name = "text"
			end
			
			if (featureindex[name] ~= nil) then
				for a,b in ipairs(featureindex[name]) do
					local conds = b[2]
					
					if testcond(conds,unitid) then
						table.insert(unitrules, b)
					end
				end
	
				local isice = isthis(unitrules,"melt")
				local ismove = isthis(unitrules,"move")
				local isyou = isthis(unitrules,"you")
				local isyou2 = isthis(unitrules,"you2")
				local ispush = isthis(unitrules,"push")
				local ispull = isthis(unitrules,"pull")
				local isdefeat = isthis(unitrules,"defeat")
				local iswin = isthis(unitrules,"win")
				local ismore = isthis(unitrules,"more")
				local istele = isthis(unitrules,"tele")
				local isweak = isthis(unitrules,"weak")
				local isshut = isthis(unitrules,"shut")
				local ishide = isthis(unitrules,"hide")
				local isbonus = isthis(unitrules,"bonus")
				local issink = isthis(unitrules,"sink")
				local isblue = isthis(unitrules,"blue")
				local isred = isthis(unitrules,"red")
				local isdone = isthis(unitrules,"done")
				local make = xthis(unitrules,name,"make")
				local eat = xthis(unitrules,name,"eat")
				
				if isdone then
					if (unit.flags[DEAD] == false) then
						table.insert(doned, unit.fixed)
					end
				end
				
				local sink = findfeature(nil,"is","sink")
				
				if (sink ~= nil) then
					local sank = false
					
					for a,b in ipairs(sink) do
						local sunk = findtype(b,x,y,unitid)
						
						if (#sunk > 0) then
							for c,d in ipairs(sunk) do
								if floating(d,unitid) then
									if (isgone(d) == false) and (d ~= unitid) then
										local dosink = true
										local sinksinker = true
										
										for e,f in ipairs(delthese) do
											if (f[1] == unitid) then
												dosink = false
											end
											
											if (f[1] == d) then
												sinksinker = false
											end
										end
										
										local sinkeffect = false
										
										if dosink then
											if (issafe(unitid) == false) then
												sank = true
											end
											
											if (issafe(d) == false) and sinksinker then
												table.insert(delthese, {d, 0})
												sinkeffect = true
											end
											
											if sinkeffect then
												removalsound = 3
												online = false
												local c1,c2 = getcolour(d)
												MF_particles("destroy",x,y,15,c1,c2)
											end
										end
									end
								end
							end
						end
					end
					
					if sank then
						table.insert(delthese, {unitid, 3})
						sinkeffect = true
					end
				end
				
				if isblue or isred then
					if isblue then
						MF_setcolour(unitid,1,3)
						unit.values[A] = 1
					elseif isred then
						MF_setcolour(unitid,2,2)
						unit.values[A] = 1
					end
				else
					if (unit.strings[UNITTYPE] ~= "text") and (unit.className ~= "level") then
						setcolour(unit.fixed)
						unit.values[A] = 0
					end
				end

				if issink then
					local water = findallhere(x,y)
					local sunk = false
					
					if (#water > 0) then
						for a,b in ipairs(water) do
							if floating(b,unitid) then
								if (issafe(b) == false) and (b ~= unitid) then
									local dosink = true
									
									for c,d in ipairs(delthese) do
										if (d[1] == unitid) or (d[1] == b) then
											dosink = false
										end
									end
									
									if dosink then	
										table.insert(delthese, {b, 3})
										removalsound = 3
										local c1,c2 = getcolour(unitid)
										MF_particles("destroy",x,y,15,c1,c2)
										
										if (b ~= unitid) and (issafe(unitid) == false) then
											sunk = true
										end
									end
								end
							end
						end
					end
					
					if sunk then
						table.insert(delthese, {unitid, 0})
						online = false
					end
				end
				
				if isshut then
					local open = findfeature(nil,"is","open")
					
					if (open ~= nil) then
						for i,v in ipairs(open) do
							local key = findtype(v,x,y,0)
							
							if (#key > 0) then
								local doparts = false
								for a,b in ipairs(key) do
									if (b ~= 0) and floating(b,unitid) then
										if (issafe(unitid) == false) then
											table.insert(delthese, {unitid, 4})
											doparts = true
											online = false
										end
										
										if (b ~= unitid) and (issafe(b) == false) then
											table.insert(delthese, {b, 0})
											doparts = true
										end
										
										if doparts then
											MF_particles("unlock",x,y,30,2,4)
										end
										
										break
									end
								end
							end
						end
					end
				end
				
				if (#eat > 0) and (isgone(unitid) == false) then
					for i,v in ipairs(eat) do
						--print(name .. " eat " .. v)
						local things = findtype({v,nil},x,y,unitid)
						
						if (#things > 0) then
							for a,b in ipairs(things) do
								if (issafe(b) == false) and floating(b,unitid) and (b ~= unitid) then
									table.insert(delthese, {b, 4})
									MF_particles("eat",x,y,5,0,3)
								end
							end
						end
					end
				end
				
				if isweak and (issafe(unitid) == false) then
					local stuff = findallhere(x,y)
					
					if (#stuff > 0) then
						for i,v in ipairs(stuff) do
							if floating(v,unitid) then
								local vunit = mmf.newObject(v)
								local thistype = vunit.strings[UNITTYPE]
								if (v ~= unitid) then
									MF_particles("destroy",x,y,5,0,3)
									table.insert(delthese, {unitid, 4})
									online = false
									break
								end
							end
						end
					end
				end
				
				if (isyou or isyou2) and online and (isgone(unitid) == false) then
					local win = findfeature(nil,"is","win")
					
					if (win ~= nil) then
						for a,b in ipairs(win) do
							if (b[1] ~= "empty") then
								local flag = findtype(b,x,y,0)
								
								if (#flag > 0) then
									for c,d in ipairs(flag) do
										if (isgone(d) == false) and (isgone(unitid) == false) and floating(d,unitid) then
											MF_particles("win",x,y,10,2,4)
											MF_win()
											break
										end
									end
								end
							end
						end
					end
					
					local defeat = findfeature(nil,"is","defeat")
					local alldefeat = findfeature("all","is","defeat")
					
					if (defeat ~= nil) and (alldefeat == nil) then
						for a,b in ipairs(defeat) do
							if (b[1] ~= "empty") then
								local skull = findtype(b,x,y,0)
								
								if (#skull > 0) and (issafe(unitid) == false) then
									for c,d in ipairs(skull) do
										if floating(d,unitid) then
											local kunit = mmf.newObject(d)
											local kname = kunit.strings[UNITNAME]
											
											if (kunit.strings[UNITTYPE] == "text") then
												kname = "text"
											end
											
											local weakskull = hasfeature(kname,"is","weak",d)
											
											if (weakskull == nil) or ((weakskull ~= nil) and issafe(d)) then
												MF_particles("destroy",x,y,5,0,3)
												table.insert(delthese, {unitid, 5})
											end
										end
									end
								end
							end
						end
					end
					
					local bonus = findfeature(nil,"is","bonus")
					
					if (bonus ~= nil) then
						for a,b in ipairs(bonus) do
							if (b[1] ~= "empty") then
								local flag = findtype(b,x,y,0)
								
								if (#flag > 0) then
									for c,d in ipairs(flag) do
										if (isgone(d) == false) and floating(d,unitid) then
											MF_particles("bonus",x,y,10,4,1)
											MF_bonus()
											table.insert(delthese, {d, 5})
										end
									end
								end
							end
						end
					end
					
					local ending = findfeature(nil,"is","end")
					
					if (ending ~= nil) then
						for a,b in ipairs(ending) do
							if (b[1] ~= "empty") then
								local flag = findtype(b,x,y,0)
								
								if (#flag > 0) then
									for c,d in ipairs(flag) do
										if (isgone(d) == false) and (isgone(unitid) == false) and floating(d,unitid) then
											MF_particles("unlock",x,y,10,1,4)
											MF_end(unitid,d)
											break
										end
									end
								end
							end
						end
					end
				end
				
				if isice then
					local hot = findfeature(nil,"is","hot")
					
					if (hot ~= nil) then
						for a,b in ipairs(hot) do
							local lava = findtype(b,x,y,0)
						
							if (#lava > 0) and (issafe(unitid) == false) then
								for c,d in ipairs(lava) do
									if floating(d,unitid) and online then
										MF_particles("smoke",x,y,5,0,1)
										table.insert(delthese, {unitid, 5})
										online = false
										break
									end
								end
							end
						end
					end
				end
				
				if isdefeat and online then
					local player = findfeature(nil,"is","you")
					local player2 = findfeature(nil,"is","you2")
					
					if (player == nil) then
						player = {}
					end
					
					if (player2 ~= nil) then
						for i,v in ipairs(player2) do
							table.insert(player, v)
						end
					end
					
					if (player ~= nil) then
						for i,v in ipairs(player) do
							local skull = findtype(v,x,y,0)
							
							if (#skull > 0) then
								for a,b in ipairs(skull) do
									if (b ~= 0) and (issafe(b) == false) and floating(b,unitid) then
										MF_particles("destroy",x,y,5,0,3)
										table.insert(delthese, {b, 5})
									end
								end
							end
						end
					end
				end
				
				if iswin and online then
					local player = findfeature(nil,"is","you")
					local player2 = findfeature(nil,"is","you2")
					
					if (player == nil) then
						player = {}
					end
					
					if (player2 ~= nil) then
						for i,v in ipairs(player2) do
							table.insert(player, v)
						end
					end
					
					if (player ~= nil) then
						for a,b in ipairs(player) do
							local flag = findtype(b,x,y,0)
							
							if (#flag > 0) then
								for c,d in ipairs(flag) do
									if (isgone(d) == false) and (isgone(unitid) == false) and floating(d,unitid) then
										MF_particles("win",x,y,10,2,4)
										MF_win()
										break
									end
								end
							end
						end
					end
				end
				
				if isbonus and online then
					local player = findfeature(nil,"is","you")
					local player2 = findfeature(nil,"is","you2")
					
					if (player == nil) then
						player = {}
					end
					
					if (player2 ~= nil) then
						for i,v in ipairs(player2) do
							table.insert(player, v)
						end
					end
					
					if (player ~= nil) then
						for a,b in ipairs(player) do
							local flag = findtype(b,x,y,0)
							
							if (#flag > 0) then
								for c,d in ipairs(flag) do
									if (isgone(d) == false) and floating(d,unitid) then
										MF_particles("bonus",x,y,10,4,1)
										MF_bonus()
										
										if (issafe(unitid) == false) then
											table.insert(delthese, {unitid, 5})
											online = false
										end
									end
								end
							end
						end
					end
				end
				
				if (#make > 0) and (isgone(unitid) == false) and (small == false) then
					local thingshere = findallhere(x,y)
					
					for i,v in ipairs(make) do
						local domake = true
						local exists = false
						
						if (v ~= "text") and (v ~= "all") then
							for b,mat in pairs(objectlist) do
								if (b == v) then
									exists = true
								end
							end
						else
							exists = true
						end
						
						if exists then
							if (#thingshere > 0) then
								for a,b in ipairs(thingshere) do
									local thing = mmf.newObject(b)
									local thingname = thing.strings[UNITNAME]
									
									if (thingname == v) or ((thing.strings[UNITTYPE] == "text") and (v == "text")) then
										domake = false
									end
								end
							end
							
							if domake then
								if (v ~= "empty") and (v ~= "all") and (v ~= "text") then
									create(v,x,y,dir,x,y)
								elseif (v == "all") then
									for b,mat in pairs(objectlist) do
										local ishere = findtype(b,x,y,0)
										
										if (#ishere == 0) then
											create(b,x,y,dir,x,y)
										end
									end
								elseif (v == "text") then
									if (name ~= "empty") and (name ~= "text") and (name ~= "all") then
										create("text_" .. name,x,y,dir,x,y)
										updatecode = 1
									end
								end
							end
						end
					end
				end
				
				if ismore and (isgone(unitid) == false) and (small == false) then
					local doblocks = {}
					
					for i=1,4 do
						local drs = ndirs[i]
						ox = drs[1]
						oy = drs[2]
						
						local valid = true
						local obs = findobstacle(x+ox,y+oy)
						local tileid = x+ox + (y+oy) * roomsize
						
						if (#obs > 0) then
							for a,b in ipairs(obs) do
								if (b == -1) then
									valid = false
								elseif (b ~= 0) and (b ~= -1) then
									local bunit = mmf.newObject(b)
									local obsname = bunit.strings[UNITNAME]
									local obstype = bunit.strings[UNITTYPE]
									
									if (obstype == "text") then
										obsname = "text"
									end
									
									local obsstop = hasfeature(obsname,"is","stop",b)
									local obspush = hasfeature(obsname,"is","push",b)
									local obspull = hasfeature(obsname,"is","pull",b)
									local obsyou = hasfeature(obsname,"is","you",b) or hasfeature(obsname,"is","you2",b)
									
									if (obsstop ~= nil) or (obspush ~= nil) or (obsyou ~= nil) or (obspull ~= nil) or (obsname == name) or (obstype == "text") then
										valid = false
									end
									
									--valid = false
								end
							end
						end
						
						if valid then
							local newunit = copy(unitid,x+ox,y+oy)
							table.insert(doblocks, newunit)
							--[[
							if isgone(newunit) then
								delete(newunit,1)
							end
							]]--
						end
					end
					
					if (#doblocks > 0) then
						block(doblocks,true)
					end
				end
				
				if istele and (small == false) and (isgone(unitid) == false) then
					local targets = findallhere(x,y)
					local telethis = false
					local telethisx,telethisy = 0,0
					local swaptele = {}
					
					if (#targets > 0) then
						for i,v in ipairs(targets) do
							local vunit = mmf.newObject(v)
							local thistype = vunit.strings[UNITTYPE]
							if (isgone(v) == false) and floating(v,unitid) then
								local targetname = vunit.strings[UNITNAME]
								if (objectdata[v] == nil) then
									objectdata[v] = {}
								end
								
								local odata = objectdata[v]
								
								if (odata.tele == nil) then
									if (targetname ~= name) and (v ~= unitid) then
										local teles = findall({name})
										
										if (#teles > 1) then
											local teletargets = {}
											local targettele = 0
											
											for a,b in ipairs(teles) do
												local tele = mmf.newObject(b)
												
												if (b ~= unit) then
													table.insert(teletargets, b)
												end
											end
											
											if (#teletargets > 0) then
												targettele = teletargets[math.random(#teletargets)]
												local limit = 0
												
												while (targettele == unitid) and (limit < 10) do
													targettele = teletargets[math.random(#teletargets)]
													limit = limit + 1
												end
												
												odata.tele = 1
												
												local tele = mmf.newObject(targettele)
												local tx,ty = tele.values[XPOS],tele.values[YPOS]
												
												MF_particles("glow",vunit.values[XPOS],vunit.values[YPOS],5,1,4)
												
												local vunitswap = hasfeature(vunit.strings[UNITNAME],"is","swap",v)
												local imswap = hasfeature(name,"is","swap",unit.fixed)
											
												update(v,tx,ty)
												
												if (vunitswap ~= nil) or (imswap ~= nil) then
													table.insert(swaptele, targettele)
												end
												
												MF_particles("glow",tx,ty,5,1,4)
												MF_sound(2)
											end
										end
									end
								end
							end
						end
					end
					
					if (#swaptele > 0) then
						for a,b in ipairs(swaptele) do
							update(b,x,y)
						end
					end
				end
				
				if ishide and unit.visible then
					unit.visible = false
					addundo({"visibility",name,id,0})
				end
			else
				if (unit.visible == false) then
					unit.visible = true
					addundo({"visibility",name,id,1})
				end
				
				if (unit.values[A] ~= 0) then
					setcolour(unit.fixed)
					unit.values[A] = 0
				end
			end
		else
			break
		end
	end
	
	for i,unitid in ipairs(doned) do
		local unit = mmf.newObject(unitid)
		unit.values[FLOAT] = 2
		unit.values[EFFECTCOUNT] = math.random(-10,10)
		unit.flags[DEAD] = true
		
		print(unit.strings[UNITNAME] .. ", " .. tostring(unit.values[ID]))
		
		delunit(unitid)
		
		addundo({"done",name,unit.values[XPOS],unit.values[YPOS],unit.values[DIR],unit.values[ID],unit.fixed})
	end
	
	if (#delthese > 0) then
		for i,v in ipairs(delthese) do
			local uid = v[1]
			if (deleted[uid] == nil) then
				delete(uid,v[2],removalsound)
				deleted[uid] = 1
			end
		end
	end
end

function startblock()
	if (featureindex["level"] ~= nil) then
		MF_levelrotation(0)
		for i,v in ipairs(featureindex["level"]) do
			local rule = v[1]
			local conds = v[2]
			
			if (#conds == 0) then
				if (rule[1] == "level") and (rule[2] == "is") then
					if (rule[3] == "down") then
						maprotation = 0
						MF_levelrotation(0)
					elseif (rule[3] == "right") then
						maprotation = 90
						MF_levelrotation(90)
					elseif (rule[3] == "up") then
						maprotation = 180
						MF_levelrotation(180)
					elseif (rule[3] == "left") then
						maprotation = 270
						MF_levelrotation(270)
					end
				end
			end
		end
	end
	
	for i,unit in ipairs(units) do
		local name = unit.strings[UNITNAME]
		local unitid = unit.fixed
		local unitrules = {}
		
		if (unit.strings[UNITTYPE] == "text") then
			name = "text"
		end
		
		if (featureindex[name] ~= nil) then
			for a,b in ipairs(featureindex[name]) do
				local conds = b[2]
				
				if testcond(conds,unitid) then
					table.insert(unitrules, b)
				end
			end
			
			local ishide = isthis(unitrules,"hide")
			local isfollow = xthis(unitrules,name,"follow")
			local isfloat = isthis(unitrules,"float")
			local sleep = isthis(unitrules,"sleep")
			local isred = isthis(unitrules,"red")
			local isblue = isthis(unitrules,"blue")
			
			if ishide then
				if unit.visible then
					unit.visible = false
				end
			end
			
			if isfloat then
				unit.values[FLOAT] = 1
			end
			
			if sleep then
				if (unit.values[TILING] == 2) or (unit.values[TILING] == 3) then
					unit.values[VISUALDIR] = -1
					unit.direction = ((unit.values[DIR] * 8 + unit.values[VISUALDIR]) + 32) % 32
				end
			end
			
			if isblue then
				MF_setcolour(unitid,1,3)
				unit.values[A] = 1
			end
			
			if isred then
				MF_setcolour(unitid,2,2)
				unit.values[A] = 1
			end
			
			if (#isfollow > 0) then
				local x,y = unit.values[XPOS],unit.values[YPOS]
				local distance = 9999
				local targetdir = -1
				
				for i,v in ipairs(isfollow) do
					local these = findall({v})
					
					if (#these > 0) then
						for a,b in ipairs(these) do
							if (b ~= unit.fixed) then
								local funit = mmf.newObject(b)
								
								local fx,fy = funit.values[XPOS],funit.values[YPOS]
								
								local xdir = fx-x
								local ydir = fy-y
								local dist = math.abs(xdir) + math.abs(ydir)
								local fdir = -1
								
								if (math.abs(xdir) <= math.abs(ydir)) then
									if (ydir >= 0) then
										fdir = 3
									else
										fdir = 1
									end
								else
									if (xdir >= 0) then
										fdir = 0
									else
										fdir = 2
									end
								end
								
								if (dist < distance) then
									distance = dist
									targetdir = fdir
								end
							end
						end
					end
				end
				
				if (targetdir >= 0) then
					updatedir(unit.fixed,targetdir)
				end
			end
		end
	end
end

function keyblock()
	local unlocked = false
	local things = {}
	local donethings = {}
	
	if (featureindex["level"] ~= nil) then
		for i,v in ipairs(featureindex["level"]) do
			table.insert(things, v)
		end
	end
	
	if (#things > 0) then
		for i,rules in ipairs(things) do
			local rule = rules[1]
			local conds = rules[2]
			
			if (#conds == 0) then
				local action = rule[3]
				
				if (action == "win") then
					local yous = findfeature(nil,"is","you")
					local yous2 = findfeature(nil,"is","you2")
					
					if (yous == nil) then
						yous = {}
					end
					
					if (yous2 ~= nil) then
						for i,v in ipairs(yous2) do
							table.insert(yous, v)
						end
					end
					
					local canwin = false
					
					if (yous ~= nil) then
						for a,b in ipairs(yous) do
							local allyous = findall(b)
							
							if (#allyous > 0) then
								canwin = true
								for c,d in ipairs(allyous) do
									local unit = mmf.newObject(d)
									MF_particles("win",unit.values[XPOS],unit.values[YPOS],10,2,4)
								end
							end
						end
					end
					
					if canwin then
						MF_win()
					end
				elseif (action == "defeat") then
					local yous = findfeature(nil,"is","you")
					local yous2 = findfeature(nil,"is","you2")
					
					if (yous == nil) then
						yous = {}
					end
					
					if (yous2 ~= nil) then
						for i,v in ipairs(yous2) do
							table.insert(yous, v)
						end
					end
					
					if (yous ~= nil) then
						for a,b in ipairs(yous) do
							local allyous = findall(b)
							
							if (#allyous > 0) then
								for c,d in ipairs(allyous) do
									if (issafe(d) == false) then
										local unit = mmf.newObject(d)
										MF_particles("destroy",unit.values[XPOS],unit.values[YPOS],5,0,3)
										delete(d,2)
									end
								end
							end
						end
					end
				elseif (action == "weak") then
					for i,unit in ipairs(units) do
						local name = unit.strings[UNITNAME]
						if (unit.strings[UNITTYPE] == "text") then
							name = "text"
						end
						if (hasfeature(name,"is","float",unit.fixed) == nil) then
							destroylevel()
						end
					end
				elseif (action == "hot") then
					local melts = findfeature(nil,"is","melt")
					
					if (melts ~= nil) then
						for a,b in ipairs(melts) do
							local allmelts = findall(b)
							
							if (#allmelts > 0) then
								for c,d in ipairs(allmelts) do
									if (issafe(d) == false) then
										local unit = mmf.newObject(d)
										MF_particles("smoke",unit.values[XPOS],unit.values[YPOS],5,0,1)
										delete(d,2)
									end
								end
							end
						end
					end
				elseif (action == "melt") then
					local hots = findfeature(nil,"is","hot")
					
					if (hots ~= nil) then
						for a,b in ipairs(hots) do
							local allhots = findall(b)
							
							if (#allhots > 0) then
								destroylevel()
							end
						end
					end
				elseif (action == "open") then
					local shuts = findfeature(nil,"is","shut")
					
					if (shuts ~= nil) then
						for a,b in ipairs(shuts) do
							local allshuts = findall(b)
							
							if (#allshuts > 0) then
								destroylevel()
							end
						end
					end
				elseif (action == "shut") then
					local opens = findfeature(nil,"is","open")
					
					if (opens ~= nil) then
						for a,b in ipairs(opens) do
							local allopens = findall(b)
							
							if (#allopens > 0) then
								destroylevel()
							end
						end
					end
				elseif (action == "sink") then
					for a,unit in ipairs(units) do
						local name = unit.strings[UNITNAME]
						if (unit.strings[UNITTYPE] == "text") then
							name = "text"
						end
						if (hasfeature(name,"is","float",unit.fixed) ~= nil) then
							destroylevel()
						end
					end
				elseif (action == "shift") then
					for a,unit in ipairs(units) do
						local x,y,dir = unit.values[XPOS],unit.values[YPOS],unit.values[DIR]
						local drs = ndirs[dir+1]
						local ox = drs[1]
						local oy = drs[2]
						
						if ((unit.values[MOVED] == 0) or (unit.values[PUSHED] ~= dir)) and (issafe(unit.fixed) == false) then
							move(unit.fixed,ox,oy,dir)
						end
					end
				elseif (action == "tele") then
					for a,unit in ipairs(units) do
						local x,y = unit.values[XPOS],unit.values[YPOS]
						
						local tx,ty = math.random(1,roomsize-2),math.random(1,roomsize-2)
						
						if (issafe(unit.fixed) == false) then
							update(unit.fixed,tx,ty)
							MF_particles("glow",x,y,5,1,4)
							MF_particles("glow",tx,ty,5,1,4)
							MF_sound(2)
						end
					end
				elseif (action == "more") then
					-- MITÄ IHMETTÄ TÄSSÄ?
				elseif (action == "move") then
					-- MITÄ IHMETTÄ TÄSSÄ?
				end
			end
		end
	end
	
	if (featureindex["done"] ~= nil) then
		for i,v in ipairs(featureindex["done"]) do
			table.insert(donethings, v)
		end
	end
	
	if (#donethings > 0) then
		for i,rules in ipairs(donethings) do
			local rule = rules[1]
			local conds = rules[2]
			
			if (#conds == 0) then
				if (rule[1] == "all") and (rule[2] == "is") and (rule[3] == "done") then
					MF_allisdone()
				end
			end
		end
	end
	
	if unlocked then
		MF_sound(3)
	end
end

function musicstate()
	local playerfound = false
	
	local players = findfeature(nil,"is","you") or findfeature(nil,"is","you2")
	
	if (players ~= nil) then
		for i,v in ipairs(players) do
			local allplayers = findall(v)
			
			if (#allplayers > 0) then
				playerfound = true
			end
		end
	end
	
	if playerfound then
		MF_musicstate(0)
	else
		MF_musicstate(1)
	end
end

function findfears(unitid)
	local unit = mmf.newObject(unitid)
	
	local result,resultid = false,4
	
	local ox,oy = 0,0
	local x,y = unit.values[XPOS],unit.values[YPOS]
	
	local name = unit.strings[UNITNAME]
	if (unit.strings[UNITTYPE] == "text") then
		name = "text"
	end
	
	local feartargets = {}
	if (featureindex[name] ~= nil) then
		for i,v in ipairs(featureindex[name]) do
			local rule = v[1]
			local conds = v[2]
			
			if (rule[2] == "fear") then
				if testcond(conds,unitid) then
					table.insert(feartargets, rule[3])
				end
			end
		end
	end
	
	for i=1,4 do
		local ndrs = ndirs[i]
		ox = ndrs[1]
		oy = ndrs[2]
		
		for j,v in ipairs(feartargets) do
			local foundfears = findtype({v, nil},x+ox,y+oy,unitid)
			
			if (#foundfears > 0) then
				result = true
				resultdir = rotate(i-1)
			end
		end
	end
	
	return result,resultdir
end