leveltree = {}

function sublevel(name)
	local id = #leveltree + 1
	local parentid = #leveltree
	
	local oldparent = leveltree[parentid]
	
	local delafterthis = 0
	
	if (parentid > 0) then
		for i,v in pairs(leveltree) do
			if (v == name) then
				delafterthis = i
				
				id = i
				parentid = i-1
			end
			
			if (delafterthis > 0) and (i > delafterthis) then
				print("Removed " .. tostring(i) .. ", " .. v)
				table.remove(leveltree, i)
			end
		end
	end
	
	generaldata.strings[CURRLEVEL] = name
	leveltree[id] = name
	
	if (oldparent ~= nil) then
		generaldata.strings[PARENT] = oldparent
	else
		generaldata.strings[PARENT] = ""
	end
end

function uplevel()
	local id = #leveltree
	local parentid = #leveltree - 1
	
	local oldlevel = generaldata.strings[CURRLEVEL]
	
	if (id == 0) then
		MF_alert("Already at map root")
	end
	
	if (parentid > 1) then
		generaldata.strings[PARENT] = leveltree[parentid - 1]
	else
		generaldata.strings[PARENT] = ""
	end
	
	if (id > 1) then
		generaldata.strings[CURRLEVEL] = leveltree[parentid]
	else
		generaldata.strings[CURRLEVEL] = ""
	end
	
	table.remove(leveltree, id)
	
	return oldlevel
end

function changelevel(newlevel)
	local id = #leveltree
	local parentid = #leveltree - 1
	
	local oldlevel = generaldata.strings[CURRLEVEL]
	
	if (id == 0) then
		MF_alert("Already at map root")
	end
	
	if (id > 0) then
		leveltree[id] = newlevel
	end

	generaldata.strings[CURRLEVEL] = newlevel
	
	return oldlevel
end

function storeleveltree(tree)
	leveltree = {}
	
	for i,v in ipairs(tree) do
		table.insert(leveltree, v)
	end
	
	local current = leveltree[#leveltree]
	local parent = ""
	
	if (#leveltree > 1) then
		parent = leveltree[#leveltree - 1]
	end
	
	generaldata.strings[CURRLEVEL] = current
	generaldata.strings[PARENT] = parent
end

function unlockeffect(dataid,cursorid)
	local data = mmf.newObject(dataid)
	local cursor = mmf.newObject(cursorid)
	
	if (hiddenmap == nil) then
		hiddenmap = {}
		hiddenmap.unlocks = {}
		hiddenmap.start = {cursor.values[XPOS],cursor.values[YPOS]}
	end
	
	local start = hiddenmap.start
	local x,y = start[1],start[2]
	
	data.values[UNLOCKTIMER] = data.values[UNLOCKTIMER] + 1
	local timer = data.values[UNLOCKTIMER]
	local unlock = data.values[UNLOCK]
	
	if (unlock == 2) then
		if (timer == 10) then
			MF_playsound("roll")
		end
		
		if (timer > 10) and (timer < 140) and (timer % 2 == 0) then
			particles("unlock",x,y,2,{2, 4})
		end
		
		if (timer == 140) then
			generaldata.values[SHAKE] = 15
			generaldata.values[IGNORE] = 0
			particles("smoke",x,y,20,{0, 3})
			
			local prizeid = MF_specialcreate("Prize")
			local prize = mmf.newObject(prizeid)
			
			prize.layer = 2
			prize.values[ONLINE] = 1
			prize.values[XPOS] = Xoffset + x * tilesize + tilesize * 0.5
			prize.values[YPOS] = Yoffset + y * tilesize + tilesize * 0.5
			prize.values[YVEL] = -20
			prize.scaleX = 0.1
			prize.scaleY = 0.1
			
			MF_playsound("winnery_fast")
			MF_playsound("pop3b")
			MF_stopsound("roll")
		end
		
		if (timer == 180) then
			local hidden,hiddenmap_ = checkhidden(x,y)
			
			if hidden then
				hiddenmap = hiddenmap_
				hiddenmap.unlocks = {}
				hiddenmap.start = {x,y}				
				
				data.values[UNLOCK] = 3
				data.values[UNLOCKTIMER] = -25
				return
			end
		end
		
		local upoint = 200
		local ox,oy = 0,0
		
		if (timer == upoint) then
			ox = 0
			oy = -1
			local unlockables = findallhere(x+ox,y+oy)
			local found = false
			
			if (#unlockables > 0) then
				for i,unitid in ipairs(unlockables) do
					local unit = mmf.newObject(unitid)
					if (unit.values[COMPLETED] == 1) then
						unit.values[COMPLETED] = 2
						found = true
					end
				end
			end
			
			if found then
				particles("hot",x+ox,y+oy,10,{0, 3})
				MF_playsound("whoosh_alt" .. tostring(math.random(1,5)))
			else
				timer = timer + 15
			end
		end
		
		if (timer == upoint + 15) then
			ox = -1
			oy = 0
			local unlockables = findallhere(x+ox,y+oy)
			local found = false
			
			if (#unlockables > 0) then
				for i,unitid in ipairs(unlockables) do
					local unit = mmf.newObject(unitid)
					if (unit.values[COMPLETED] == 1) then
						unit.values[COMPLETED] = 2
						found = true
					end
				end
			end
			
			if found then
				particles("hot",x+ox,y+oy,10,{0, 3})
				MF_playsound("whoosh_alt" .. tostring(math.random(1,5)))
			else
				timer = timer + 15
			end
		end
		
		if (timer == upoint + 30) then
			ox = 0
			oy = 1
			local unlockables = findallhere(x+ox,y+oy)
			local found = false
			
			if (#unlockables > 0) then
				for i,unitid in ipairs(unlockables) do
					local unit = mmf.newObject(unitid)
					if (unit.values[COMPLETED] == 1) then
						unit.values[COMPLETED] = 2
						found = true
					end
				end
			end
			
			if found then
				particles("hot",x+ox,y+oy,10,{0, 3})
				MF_playsound("whoosh_alt" .. tostring(math.random(1,5)))
			else
				timer = timer + 15
			end
		end
		
		if (timer == upoint + 45) then
			ox = 1
			oy = 0
			local unlockables = findallhere(x+ox,y+oy)
			local found = false
			
			if (#unlockables > 0) then
				for i,unitid in ipairs(unlockables) do
					local unit = mmf.newObject(unitid)
					if (unit.values[COMPLETED] == 1) then
						unit.values[COMPLETED] = 2
						found = true
					end
				end
			end
			
			if found then
				particles("hot",x+ox,y+oy,10,{0, 3})
				MF_playsound("whoosh_alt" .. tostring(math.random(1,5)))
			end
		end
		
		if (timer == upoint + 50) then
			if (matches == nil) then
				if (data.values[MAPCLEAR] == 0) then
					data.values[UNLOCK] = 0
					data.values[UNLOCKTIMER] = 0
					generaldata.values[IGNORE] = 0
					hiddenmap = nil
				else
					data.values[MAPCLEAR] = 0
					data.values[UNLOCK] = 4
					data.values[UNLOCKTIMER] = 0
				end
			else
				data.values[UNLOCK] = 5
				data.values[UNLOCKTIMER] = 0
			end
		end
	elseif (unlock == 3) then
		local docheck = timer % 10
		
		if (docheck == 0) and (timer > 0) then
			local newstuff = {}
			local unlocks = {}
			local currtype = 0
			
			if (#hiddenmap > 0) then
				for i,v in ipairs(hiddenmap) do
					local unit = mmf.newObject(v)
					unit.values[COMPLETED] = math.max(unit.values[COMPLETED], 1)
					local ux,uy = unit.values[XPOS],unit.values[YPOS]
					
					if (unit.strings[UNITNAME] ~= "path") then
						--MF_savelevel(v, unit.values[COMPLETED])
						currtype = 0
					else
						--MF_savepath(v)
						table.insert(unlocks, {ux, uy})
						currtype = 1
					end
					
					MF_playsound("whoosh_quiet" .. tostring(math.random(1,5)))
					particles("glow",ux,uy,5,{1, 2})
					
					local th,things = checkhidden(ux,uy)
					
					if th then
						for c,d in ipairs(things) do
							local dunit = mmf.newObject(d)
							
							local addthis = false
							
							if ((dunit.strings[UNITNAME] ~= "path") and (currtype == 0)) or ((ux == x) and (uy == y)) then
								addthis = true
							elseif (dunit.strings[UNITNAME] == "path") and (currtype == 1) then
								addthis = true
								
								if (unit.strings[UNITNAME] == "path") and (unit.values[PATH_GATE] > 0) then
									addthis = false
								end
							end
							
							if addthis then
								table.insert(newstuff, d)
							end
						end
					end
				end
			end
			
			if (#hiddenmap.unlocks > 0) then
				for i,v in ipairs(hiddenmap.unlocks) do
					local ux,uy = v[1],v[2]
					unlocklevels(ux,uy,true)
				end
			end
			
			hiddenmap = {}
			hiddenmap.unlocks = {}
			hiddenmap.start = {x,y}
			
			if (#newstuff > 0) then
				for i,v in ipairs(newstuff) do
					table.insert(hiddenmap, v)
				end
			end
			
			if (#unlocks > 0) then
				for i,v in ipairs(unlocks) do
					table.insert(hiddenmap.unlocks, v)
				end
			end
			
			if (#newstuff == 0) and (#unlocks == 0) then
				hiddenmap = nil
				data.values[UNLOCK] = 2
				data.values[UNLOCKTIMER] = 180
			end
		end
	end	
end

function unlocklevels(x,y,visuals_,reveal_)
	local ox,oy = 0,0
	local visuals = false
	local reveal = false
	
	if (visuals_ ~= nil) then
		visuals = visuals_
	end
	
	if (reveal_ ~= nil) then
		reveal = reveal_
	end
	
	for a=1,4 do
		local ndrs = ndirs[a]
		ox = ndrs[1]
		oy = ndrs[2]
		
		local levels = findallhere(x+ox,y+oy)
		
		if (#levels > 0) then
			for i,v in ipairs(levels) do
				local unit = mmf.newObject(v)
				
				if (unit.values[COMPLETED] < 2) and (string.len(unit.strings[LEVELFILE]) > 0) then
					if (visuals == false) then
						if (unit.strings[UNITNAME] ~= "path") then
							MF_savelevel(v, 2)
							--MF_store("save",generaldata.strings[WORLD],unit.strings[LEVELFILE],"2")
						else
							if (unit.values[PATH_GATE] == 0) then
								MF_savepath(v, 3)
								--MF_alert("Saved path in hiddendata (3)")
							else
								MF_savepath(v, 1)
								--MF_alert("Saved path in hiddendata (1)")
							end
						end
					end
					
					if visuals or reveal then
						unit.values[COMPLETED] = 2
						
						if (unit.strings[UNITNAME] == "path") and (unit.values[PATH_GATE] > 0) then
							unit.values[COMPLETED] = 1
						end
					end
					
					if visuals then
						particles("hot",x+ox,y+oy,10,{0, 3})
						MF_playsound("whoosh_alt" .. tostring(math.random(1,5)))
					end
				end
			end
		end
	end
end

function checkhidden(x,y)
	local ox,oy = 0,0
	local result = {}
	local current = {}
	local hidden = false
	
	for a=1,4 do
		local ndrs = ndirs[a]
		ox = ndrs[1]
		oy = ndrs[2]
		
		local levels = findallhere(x+ox,y+oy,nil,true)
		
		if (#levels > 0) then
			for i,v in ipairs(levels) do
				local unit = mmf.newObject(v)
				
				if (unit.values[COMPLETED] == 0) and (string.len(unit.strings[LEVELFILE]) > 0) then
					table.insert(result, v)
					hidden = true
				end
			end
		end
	end
	
	return hidden,result
end

function prizeget(id)
	local unit = mmf.newObject(id)
	local x = unit.x / tilesize
	local y = unit.y / tilesize
	particles("glow",x,y,20,{0,3})
end

function hiddendata(x,y,reveal_)
	local more = true
	local reveal = false
	
	if (reveal_ ~= nil) then
		reveal = reveal_
	end
	
	local hiddens = {}
	local alreadydone = {}
	
	local cx,cy = x,y
	
	local currtype = 0
	
	while more do
		local id = cx + cy * roomsize
		
		if (alreadydone[id] == nil) then
			local current = findallhere(cx,cy,nil,true)
			local current_gate = false
			
			if (#current > 0) then
				for i,v in ipairs(current) do
					local unit = mmf.newObject(v)
					
					if (unit.strings[UNITNAME] == "path") then
						currtype = 1
						
						if (unit.values[PATH_GATE] > 0) then
							current_gate = true
						end
						
						if (cx == x) and (cy == y) then
							current_gate = false
						end
					else
						currtype = 0
					end
				end
			end
			
			if (cx == x) and (cy == y) then
				currtype = -1
			end
			
			local hidden,result = checkhidden(cx,cy)
			
			if (#result > 0) then
				if (current_gate == false) then
					for i,v in ipairs(result) do
						local unit = mmf.newObject(v)

						table.insert(hiddens, {v, currtype})
					end
				end
			end
		end
		
		alreadydone[id] = 1
		
		if (#hiddens > 0) then
			local unitid = hiddens[1][1]
			local unit = mmf.newObject(unitid)
			currtype = hiddens[1][2]
			table.remove(hiddens, 1)
			
			if (unit.strings[UNITNAME] ~= "path") then
				if (currtype == 0) or (currtype == -1) then
					MF_savelevel(unitid, 1)
					cx,cy = unit.values[XPOS],unit.values[YPOS]
					
					if reveal then
						unit.values[COMPLETED] = 1
					end
				end
			else
				if (currtype == 1) or (currtype == -1) then
					if (unit.values[PATH_GATE] == 0) then
						MF_savepath(unitid, 3)
						--MF_alert("Saved path in hiddendata (3)")
					else
						MF_savepath(unitid, 1)
						--MF_alert("Saved path in hiddendata (1)")
					end
					cx,cy = unit.values[XPOS],unit.values[YPOS]
					
					unlocklevels(cx,cy,nil,reveal)
					
					if reveal then
						unit.values[COMPLETED] = 1
					end
				end
			end
		else
			more = false
		end
	end
end

function getleveltree()
	local result = ""
	
	if (#leveltree > 0) then
		for i,v in ipairs(leveltree) do
			if (i < #leveltree) then
				result = result .. v .. ","
			else
				result = result .. v
			end
		end
	else
		print("Leveltree is empty?")
	end
	
	return result
end

function openlevels()
	local levels = {}
	
	for i,unit in ipairs(units) do
		if (unit.className == "level") then
			table.insert(levels, unit.fixed)
		end
	end
	
	if (#levels > 0) then
		for i,unitid in ipairs(levels) do
			local unit = mmf.newObject(unitid)
			
			if (unit.values[COMPLETED] == 3) then
				local x,y = unit.values[XPOS],unit.values[YPOS]
				
				unlocklevels(x,y,nil,true)
				
				hiddendata(x,y,true)
			end
		end
	end
end

function addpath(id)
	local unitid = MF_create("path")
	local unit = mmf.newObject(unitid)
	
	local i = tostring(id)
	
	unit.values[ONLINE] = 1
	unit.values[XPOS] = MF_read("level","paths",i .. "X")
	unit.values[YPOS] = MF_read("level","paths",i .. "Y")
	unit.values[DIR] = MF_read("level","paths",i .. "dir")
	unit.values[PATH_STYLE] = MF_read("level","paths",i .. "style")
	unit.values[PATH_GATE] = MF_read("level","paths",i .. "gate")
	unit.values[PATH_REQUIREMENT] = MF_read("level","paths",i .. "requirement")
	
	local world = generaldata.strings[WORLD]
	local pathid = generaldata.strings[CURRLEVEL] .. "," .. tostring(unit.values[XPOS]) .. "," .. tostring(unit.values[YPOS])
	
	local completed_save = tonumber(MF_read("save",world,pathid)) or 0
	
	unit.values[COMPLETED] = math.max(unit.values[COMPLETED], completed_save)
	
	if (unit.values[PATH_STYLE] > 0) then
		unit.values[COMPLETED] = 3
	end
	
	unit.strings[PATH_OBJECT] = MF_read("level","paths",i .. "object")
	
	unit.x = Xoffset + unit.values[XPOS] * tilesize + tilesize * 0.5
	unit.y = Yoffset + unit.values[YPOS] * tilesize + tilesize * 0.5
	
	unit.visible = false

	table.insert(paths, unitid)
end

function loadlevelcompletion()
	local world = generaldata.strings[WORLD]
	
	for i,unit in ipairs(units) do
		local level = unit.strings[LEVELFILE]
		if (string.len(level) > 0) then
			local completion = tonumber(MF_read("save",world,level)) or 0
			
			if (completion > 0) then
				unit.values[COMPLETED] = completion
			end
		end
	end
	
	for i,unitid in ipairs(paths) do
		local unit = mmf.newObject(unitid)
		local level = unit.strings[LEVELFILE]
		local x,y = unit.values[XPOS],unit.values[YPOS]
		
		if (string.len(level) > 0) then
			local completion = tonumber(MF_read("save",world,level .. "," .. tostring(x) .. "," .. tostring(y))) or 0
			
			if (completion > 0) then
				unit.values[COMPLETED] = completion
			end
		end
	end
end

function revealpaths(dataid)
	local gdata = mmf.newObject(dataid)
	local world = generaldata.strings[WORLD]
	local level = generaldata.strings[CURRLEVEL]
	
	for i,unitid in ipairs(paths) do
		local unit = mmf.newObject(unitid)
		local x,y = unit.values[XPOS],unit.values[YPOS]
		
		if (unit.values[COMPLETED] > 0) then
			local newid = MF_create(unit.strings[PATH_OBJECT])
			local new = mmf.newObject(newid)
			
			new.values[ONLINE] = 1
			new.values[XPOS] = unit.values[XPOS]
			new.values[YPOS] = unit.values[YPOS]
			new.x = Xoffset + new.values[XPOS] * tilesize + tilesize * 0.5
			new.y = Yoffset + new.values[YPOS] * tilesize + tilesize * 0.5
			new.values[DIR] = unit.values[DIR]
			new.values[COMPLETED] = 3 - math.min(unit.values[PATH_GATE], 1) * 2
			new.strings[LEVELFILE] = ""
			
			addunit(newid)
			
			local prizes = tonumber(MF_read("save",world .. "_prize","total")) or 0
			local clears = tonumber(MF_read("save",world .. "_clears","total")) or 0
			local status = tonumber(MF_read("save",world,level .. "," .. tostring(x) .. "," .. tostring(y))) or 0

			if (unit.values[PATH_GATE] == 1) and (prizes >= unit.values[PATH_REQUIREMENT]) then
				if (status < 3) then
					new.values[COMPLETED] = -2
				else
					new.values[COMPLETED] = 3
					opengate(newid)
				end
			elseif (unit.values[PATH_GATE] == 2) and (clears >= unit.values[PATH_REQUIREMENT]) then
				if (status < 3) then
					new.values[COMPLETED] = -3
				else
					new.values[COMPLETED] = 3
					opengate(newid)
				end
			end
			
			if (new.values[TILING] == 1) then
				dynamic(newid,{"level"})
			end
				
			unit.flags[DEAD] = true
			MF_cleanremove(unitid)
			
			for a,b in pairs(paths) do
				if (b == unitid) then
					table.remove(paths, a)
				end
			end
		end
	end
end

function opengate(unitid)
	local unit = mmf.newObject(unitid)
	
	local pathobject = editor.strings[PATH_OBJECT]
	
	local newid = MF_create(pathobject)
	local new = mmf.newObject(newid)
	
	new.values[ONLINE] = 1
	new.values[XPOS] = unit.values[XPOS]
	new.values[YPOS] = unit.values[YPOS]
	new.x = Xoffset + new.values[XPOS] * tilesize + tilesize * 0.5
	new.y = Yoffset + new.values[YPOS] * tilesize + tilesize * 0.5
	new.values[DIR] = unit.values[DIR]
	new.values[COMPLETED] = 3
	new.strings[LEVELFILE] = ""
	
	addunit(newid)
	
	if (new.values[TILING] == 1) then
		dynamic(newid,{"level"})
	end
	
	delunit(unitid)
	MF_cleanremove(unitid)
end

function mapunlock(dataid,cursorid)
	local data = mmf.newObject(dataid)
	local cursor = mmf.newObject(cursorid)
	local world = generaldata.strings[WORLD]
	local level = generaldata.strings[CURRLEVEL]
	
	local INEDITOR = 15
	
	local MODE = 3
	local TRANSITION = 18
	local TRANSITIONREASON = 20
	
	data.values[UNLOCKTIMER] = data.values[UNLOCKTIMER] + 1
	local timer = data.values[UNLOCKTIMER]
	local unlock = data.values[UNLOCK]
	
	if (unlock == 4) then
		if (timer == 5) then
			local levels = MF_filelist("Data/Worlds/" .. world .. "/","*.ld")
			
			matches = {}
			matches.origin = level
			matches.previous = data.strings[1]
			
			if (#levels > 0) then
				for i,v in ipairs(levels) do
					local levelname = string.sub(v, 1, string.len(v) - 3)
					MF_setfile("level","Data/Worlds/" .. world .. "/" .. v)
					
					local levelshere = tonumber(MF_read("level","general","levels"))
					
					if (levelshere > 0) then
						for j=0,levelshere-1 do
							if (MF_read("level","levels",tostring(j) .. "file") == level) then
								table.insert(matches, levelname)
							end
						end
					end
				end
			end
			
			MF_setfile("level","Data/Worlds/" .. world .. "/" .. level .. ".ld")
		end
		
		if (timer == 10) and (#matches == 0) then
			generaldata.values[IGNORE] = 0
			data.values[MAPCLEAR] = 0
			data.values[UNLOCK] = 0
			data.values[UNLOCKTIMER] = 0
		end
	end
	
	if (unlock == 4) or (unlock == 5) then
		if (timer == 10) and (#matches > 0) then
			data.values[UNLOCK] = 5
			unlock = data.values[UNLOCK]
		end
	end
	
	if (unlock == 5) then
		if (timer == -20) and (generaldata.values[MODE] == 0) and (editor.values[INEDITOR] == 0) then
			for i,unit in ipairs(units) do
				if (unit.strings[LEVELFILE] == matches.origin) then
					if (hiddenmap == nil) then
						hiddenmap = {}
						hiddenmap.unlocks = {}
					end
					
					hiddenmap.start = {unit.values[XPOS],unit.values[YPOS]}
					
					generaldata.values[SHAKE] = 15
					
					data.values[UNLOCK] = 2
					data.values[UNLOCKTIMER] = 141
					
					unit.direction = 16
					unit.values[COMPLETED] = 3

					particles("smoke",unit.values[XPOS],unit.values[YPOS],20,{0,3})
				end
			end
		end
		
		if (timer == 10) and (#matches == 0) then
			data.values[UNLOCK] = 6
			unlock = data.values[UNLOCK]
		end
		
		if (timer == 60) then
			generaldata.values[TRANSITIONREASON] = 10
			generaldata.values[TRANSITION] = 0
			MF_loop("transition",1)
		end
	end
	
	if (unlock == 6) then
		if (timer == 80) then
			generaldata.values[TRANSITIONREASON] = 10
			generaldata.values[TRANSITION] = 0
			MF_loop("transition",1)
		end
	end
	
	if (unlock == 5) or (unlock == 6) then
		if (generaldata.values[TRANSITION] == 1) and (generaldata.values[TRANSITIONREASON] == 10) and (generaldata.values[MODE] == 0) and (editor.values[INEDITOR] == 0) then
			cursor.visible = false
			mapunlock_transition(dataid,cursorid)
		end
	end
end

function mapunlock_transition(dataid,cursorid)
	local data = mmf.newObject(dataid)
	local cursor = mmf.newObject(cursorid)
	local world = generaldata.strings[WORLD]
	
	cursor.visible = false
	
	local PREVIOUS = 1
	
	local TRANSITION = 18
	local TRANSITIONREASON = 20
	
	local unlock = data.values[UNLOCK]
	
	if (unlock == 5) then
		cursor.visible = false
		
		generaldata.values[TRANSITION] = 0
		generaldata.values[TRANSITIONREASON] = 0

		generaldata.strings[CURRLEVEL] = matches[1]
		table.remove(matches, 1)
		
		data.values[UNLOCKTIMER] = -90
		
		MF_loop("clear",1)
		MF_loop("new",1)
		
		for i,unit in ipairs(units) do
			if (unit.strings[LEVELFILE] == matches.origin) then
				unit.direction = 0
				unit.values[COMPLETED] = 2
			end
		end
	elseif (unlock == 6) then
		generaldata.values[TRANSITION] = 0
		generaldata.values[TRANSITIONREASON] = 0
		generaldata.values[IGNORE] = 0
		
		generaldata.strings[CURRLEVEL] = matches.origin
		
		cursor.visible = true
		
		data.strings[PREVIOUSLEVEL] = matches.previous
		
		MF_loop("clear",1)
		MF_loop("new",1)
		
		data.values[UNLOCK] = 0
		data.values[UNLOCKTIMER] = 0
		data.values[MAPCLEAR] = 0
		
		--[[
		for i,unit in ipairs(units) do
			if (unit.strings[LEVELFILE] == matches.previous) then
				cursor.values[XPOS] = unit.values[XPOS]
				cursor.values[YPOS] = unit.values[YPOS]
				cursor.values[ONLEVEL] = unit.fixed
			end
		end
		]]--
		
		matches = nil
	end
end

function getgates()
	gatelist = {}
	local world = generaldata.strings[WORLD]
	local level = generaldata.strings[CURRLEVEL]
	
	generaldata.values[IGNORE] = 1
	
	for i,unit in ipairs(units) do
		if (unit.values[COMPLETED] < -1) then
			table.insert(gatelist, unit.fixed)
			local x,y = unit.values[XPOS],unit.values[YPOS]
			MF_store("save",world,level .. "," .. tostring(x) .. "," .. tostring(y),"3")
				
			hiddendata(x,y,false)
		end
	end
end

function gateunlock(dataid,cursorid)
	local data = mmf.newObject(dataid)
	local cursor = mmf.newObject(cursorid)
	local world = generaldata.strings[WORLD]
	local level = generaldata.strings[CURRLEVEL]
	
	data.values[UNLOCKTIMER] = data.values[UNLOCKTIMER] + 1
	local timer = data.values[UNLOCKTIMER]
	local unlock = data.values[UNLOCK]

	if (unlock == 7) then
		for i,unitid in ipairs(gatelist) do
			local unit = mmf.newObject(unitid)
			local x,y = unit.values[XPOS],unit.values[YPOS]
			
			if (timer == 10) then
				MF_gateeffect(unitid,unit.values[PATH_GATE],unit.values[PATH_REQUIREMENT])
			end
			
			if (timer > 10) and (timer < 130) then
				if (timer % 20 == 0) then 
					if (unit.values[COMPLETED] == -2) then
						particles("bonus",x,y,2,{2, 4})
					elseif (unit.values[COMPLETED] == -3) then
						particles("bonus",x,y,2,{4, 1})
					end
				end
			elseif (timer == 130) then
				generaldata.values[SHAKE] = 15
				generaldata.values[IGNORE] = 0
				
				if (unit.values[COMPLETED] == -2) then
					particles("bonus",x,y,40,{2, 4})
				elseif (unit.values[COMPLETED] == -3) then
					particles("bonus",x,y,40,{4, 1})
				end
					
				data.values[UNLOCK] = 0
				data.values[UNLOCKTIMER] = 0
				
				particles("unlock",x,y,20,{0,3})
				
				local hidden = checkhidden(x,y)
				
				if hidden then
					data.values[UNLOCKTIMER] = 141
					data.values[UNLOCK] = 2
					
					hiddenmap = {}
					hiddenmap.unlocks = {}
					hiddenmap.start = {x,y}
				end
				
				opengate(unitid)
			end
		end
	end
end