function writetext(text,owner,xoffset,yoffset,type,centered_,layer_)
	local letterw = 10
	local letterh = 16
	local length = string.len(text)
	
	local layer = 2
	if (owner == -1) or (owner == 0) then
		layer = 3
	end
	
	if (layer_ ~= nil) then
		layer = layer_
	end
	
	local centered = false
	if (centered_ ~= nil) then
		centered = centered_
	end
	
	local x,y = 0,0
	local finalx,finaly = 0,0
	local vi = 0
	
	local c1,c2 = getuicolour("default")
	local offset = 0
	
	local letters = {}
	
	for i=1,length do
		local j = i + offset
		local letter = string.sub(text, j, j)
		
		if (letter ~= "$") then
			x = x + letterw
			vi = vi + 1
		else
			local cdata = string.sub(text, j+1, j+3)
			
			c1 = tonumber(string.sub(cdata, 1, 1))
			c2 = tonumber(string.sub(cdata, 3, 3))
			
			offset = offset + 3
		end
		
		if (centered == false) then
			finalx = x
			finaly = y
		else
			finalx = (0 - (length * 0.5) + vi) * letterw
		end
		
		if (letter ~= " ") and (letter ~= "$") then
			local tid = MF_letter(letter,owner,type,xoffset+finalx-4,yoffset+finaly,layer)
			
			if (c1 ~= 0) or (c2 ~= 3) then
				MF_setcolour(tid, c1, c2)
			end
			
			table.insert(letters, {tid, vi})
		end
		
		j = i + offset
		if (j >= length) then
			break
		end
	end
	
	if centered and ((owner == 0) or (owner == -1)) then
		for i,data in ipairs(letters) do
			local unit = mmf.newObject(data[1])
			
			finalx = (0 - (vi * 0.5) + data[2]) * letterw
			
			unit.values[XPOS] = xoffset + finalx - 4
			unit.values[YPOS] = yoffset + finaly
			unit.x = Xoffset + unit.values[XPOS]
			unit.y = Yoffset + unit.values[YPOS]
		end
	end
end

function erase(text)
	local result = string.sub(text, 1, string.len(text) - 1)
	
	return result
end

function createbutton(func,x,y,layer,xscale,yscale,text,menu,c1,c2,id_,disabled_)
	local buttonid = MF_create("Editor_editorbutton")
	local button = mmf.newObject(buttonid)
	button.x = Xoffset + x
	button.y = Yoffset + y
	button.values[ONLINE] = 1
	button.values[YORIGIN] = Yoffset + y
	button.layer = layer
	button.scaleX = xscale - (8 / tilesize)
	button.scaleY = yscale - (8 / tilesize)
	button.strings[BUTTONFUNC] = func
	writetext(text,buttonid,0,0,menu,true)
	MF_setcolour(buttonid, c1, c2)
	
	local disabled = 0
	if (disabled_ ~= nil) then
		if disabled_ then
			disabled = 1
		end
	end
	
	local id = "EditorButton"
	
	if (id_ ~= nil) then
		id = id_
	end
	
	button.strings[BUTTONID] = id
	button.values[BUTTON_DISABLED] = disabled
	
	local bits = {{"ul", -1, -1},{"u", 0, -1},{"ur", 1, -1},{"l", -1, 0},{"r", 1, 0},{"dl", -1, 1},{"d", 0, 1},{"dr", 1, 1}}
	
	for i,v in ipairs(bits) do
		local bdir = v[1]
		local bxoffset = v[2]
		local byoffset = v[3]
		
		local bid = MF_specialcreate("Editor_editorbutton_" .. bdir)
		local b = mmf.newObject(bid)
		
		b.strings[BUTTONFUNC] = func
		b.values[ONLINE] = buttonid
		b.layer = layer
		b.values[XPOS] = bxoffset
		b.values[YPOS] = byoffset
		b.values[XVEL] = xscale
		b.values[YVEL] = yscale
		b.values[YORIGIN] = Yoffset + y
		b.strings[BUTTONID] = id
		b.values[BUTTON_DISABLED] = disabled
		
		if (bdir == "u") or (bdir == "d") then
			b.scaleX = xscale - (8 / tilesize)
		elseif (bdir == "l") or (bdir == "r") then
			b.scaleY = yscale - (8 / tilesize)
		end
		
		MF_setcolour(bid, c1, c2)
	end
	
	return buttonid
end

function updatebuttontext(func,text,menu)
	local buttons = MF_getbutton(func)
	
	if (#buttons > 0) then
		for i,v in ipairs(buttons) do
			MF_buttonletterclear(v)
			
			writetext(text,v,0,0,menu,true)
		end
	end
end

function updatebuttoncolour(unitid,value)
	local c = gettoggle(value)
	
	MF_setcolour(unitid, c[1],c[2])
end

function slider(func,x,y,width,colour,colour2,id,minimum,maximum,current)
	local barid = MF_create("Editor_slider")
	local knobid = MF_create("Editor_sliderknob")
	local bar = mmf.newObject(barid)
	local knob = mmf.newObject(knobid)
	
	MF_setcolour(barid,colour[1],colour[2])
	MF_setcolour(knobid,colour2[1],colour2[2])
	
	bar.scaleX = width
	bar.x = Xoffset + x
	bar.y = Yoffset + y
	bar.layer = 2
	bar.values[ONLINE] = 1
	bar.values[YORIGIN] = Yoffset + y
	bar.strings[BUTTONFUNC] = func
	bar.strings[BUTTONID] = id
	
	bar.values[SLIDER_MIN] = minimum
	bar.values[SLIDER_CURR] = current
	bar.values[SLIDER_MAX] = maximum
	
	knob.layer = 2
	knob.values[ONLINE] = 1
	knob.values[YORIGIN] = Yoffset + y
	knob.values[SLIDERKNOB_OWNER] = barid
	knob.strings[BUTTONFUNC] = func
	knob.strings[BUTTONID] = id
end

function displaylevelname(name,layer_,group_)
	local xoffset = tilesize * 0.5
	local yoffset = tilesize * 0.5
	
	local layer = 2
	if (layer_ ~= nil) then
		layer = layer_
	end
	
	local group = "editorname"
	if (group_ ~= nil) then
		group = group_
	end

	MF_letterclear(group)
	writetext(name,-1,xoffset,yoffset,group,false,layer)
end

function submenu(menuitem)
	local currmenu = menu[1]
	table.insert(menu, 1, menuitem)
	MF_letterhide(currmenu,0)
	editor.strings[MENU] = menu[1]
	
	if (menufuncs[currmenu] ~= nil) then
		local func = menufuncs[currmenu]
		
		if (func.submenu_leave ~= nil) then
			func.submenu_leave(currmenu,menuitem)
		end
	end
	
	if (menufuncs[menuitem] ~= nil) then
		local func = menufuncs[menuitem]
		
		if (func.enter ~= nil) then
			func.enter(currmenu,menuitem)
		end
	end
	
	editor.values[SCROLLAMOUNT] = 0
end

function changemenu(menuitem)
	local currmenu = menu[1]
	MF_letterclear(currmenu,0)
	menu[1] = menuitem
	editor.strings[MENU] = menu[1]
	
	if (menufuncs[currmenu] ~= nil) then
		local func = menufuncs[currmenu]
		
		if (func.leave ~= nil) then
			func.leave(menu[2],currmenu)
		end
	end
	
	if (menufuncs[menuitem] ~= nil) then
		local func = menufuncs[menuitem]
		
		if (func.enter ~= nil) then
			func.enter(currmenu,menuitem)
		end
	end
	
	editor.values[SCROLLAMOUNT] = 0
end

function closemenu()
	local currmenu = menu[1]
	local deleted = menu[1]
	MF_letterclear(currmenu)
	table.remove(menu, 1)
	currmenu = menu[1]
	MF_letterhide(currmenu,1)
	editor.strings[MENU] = menu[1]
	if (menufuncs[deleted] ~= nil) then
		local func = menufuncs[deleted]
		
		if (func.leave ~= nil) then
			func.leave(currmenu,deleted)
		end
	end
	
	if (menufuncs[currmenu] ~= nil) then
		local func = menufuncs[currmenu]
		
		if (func.submenu_return ~= nil) then
			func.submenu_return(deleted,currmenu)
		end
	end
	
	editor.values[SCROLLAMOUNT] = 0
end

function scrollarea(list)
	local miny = 0
	local maxy = 0
	
	local scroll = 0
	
	if (#list > 0) then
		for i,unitid in ipairs(list) do
			local unit = mmf.newObject(unitid)
			
			if (unit.yTop < miny) or (miny == 0) then
				miny = unit.yTop
			end
			
			if (unit.yBottom > maxy) or (maxy == 0) then
				maxy = unit.yBottom
			end
		end
	else
		print("No list")
	end
	
	scroll = maxy - miny
	
	editor.values[SCROLLAREA] = scroll
end

function gettoggle(value_)
	local value = tonumber(value_)
	if (value == 1) then
		return colours["toggle_on"]
	else
		return colours["toggle_off"]
	end
end

function makeselection(options,choice)
	for i,option in ipairs(options) do
		local buttons = MF_getbutton(option)
		
		local selected = 0
		if (i == choice) then
			selected = 1
		end
		
		local c = gettoggle(selected)
		
		if (#buttons > 0) then
			for a,b in ipairs(buttons) do
				local unit = mmf.newObject(b)
				
				if unit.visible then
					MF_setcolour(b, c[1], c[2])
				end
			end
		end
	end
end

function creditstext(text_,id)
	local x = roomsize * tilesize * 0.5
	local y = roomsize * tilesize + id * tilesize
	
	local text = string.lower(text_)
	
	writetext(text,0,x,y,"credits",true,2)
end