function init(tilemapid,roomsize_,tilesize_,Xoffset_,Yoffset_,generaldataid)
	map = TileMap.new(tilemapid)
	generaldata = mmf.newObject(generaldataid)
	roomsize = roomsize_
	tilesize = tilesize_
	Xoffset = Xoffset_
	Yoffset = Yoffset_
	features = {}
	visualfeatures = {}
	featureindex = {}
	objectdata = {}
	units = {}
	tiledunits = {}
	codeunits = {}
	unitlists = {}
	objectlist = {}
	undobuffer = {}
	animunits = {}
	unitmap = {}
	deleted = {}
	currobjid = 0
	updatecode = 1
	doundo = true
	updateundo = true
	ruledebug = false
	maprotation = 0
	ruleids = {}
	updatelist = {}
	objectcolours = {}
	wordunits = {}
	paths = {}
	paradox = {}
	
	generatetiles()
end

function addunit(id)
	local unitid = #units + 1
	
	units[unitid] = {}
	units[unitid] = mmf.newObject(id)
	
	local unit = units[unitid]
	
	getmetadata(unit)
	
	local truename = unit.className
	
	if (changes[truename] ~= nil) then
		dochanges(id)
	end
	
	if (unit.values[ID] == -1) then
		unit.values[ID] = newid()
	end

	if (unit.values[XPOS] > 0) and (unit.values[YPOS] > 0) then
		addunitmap(id,unit.values[XPOS],unit.values[YPOS])
	end
	
	if (unit.values[TILING] == 1) then
		table.insert(tiledunits, unit.fixed)
	end
	
	if (unit.values[TILING] > 1) then
		table.insert(animunits, unit.fixed)
	end
	
	local name = unit.strings[UNITNAME]
	local name_ = unit.strings[NAME]
	
	if (unitlists[name] == nil) then
		unitlists[name] = {}
	end
	
	if (unit.strings[UNITTYPE] ~= "text") or ((unit.strings[UNITTYPE] == "text") and (unit.values[TYPE] == 0)) then
		objectlist[name_] = 1
	end
	
	table.insert(unitlists[name], unit.fixed)
	
	if (unit.strings[UNITTYPE] == "text") then
		table.insert(codeunits, unit.fixed)
		updatecode = 1
		
		if (unit.values[TYPE] == 0) then
			local matname = string.sub(unit.strings[UNITNAME], 6)
			if (unitlists[matname] == nil) then
				unitlists[matname] = {}
			end
		end
	end
	
	if (unit.strings[UNITNAME] ~= "level") then
		setcolour(unit.fixed)
	end
	
	statusblock({id})
	MF_animFrame(id,math.random(0,2))
end

function clearunits()
	units = {}
	tiledunits = {}
	codeunits = {}
	animunits = {}
	unitlists = {}
	undobuffer = {}
	unitmap = {}
	prevunitmap = {}
	currobjid = 0
	updateundo = true
	newundo()
	ruleids = {}
	objectlist = {}
	updatelist = {}
	objectcolours = {}
	wordunits = {}
	paths = {}
	paradox = {}
	hiddenmap = nil
	
	restoredefaults()
end

function smallclear()
	objectdata = {}
	deleted = {}
	updatelist = {}
end

function clear()
	features = {}
	featureindex = {}
	visualfeatures = {}
	objectdata = {}
	deleted = {}
	updatecode = 1
	updateundo = false
	ruleids = {}
	updatelist = {}
	wordunits = {}
	paradox = {}
	hiddenmap = nil
	
	collectgarbage()
end

function command(key,player_)
	local keyid = -1
	if (keys[key] ~= nil) then
		keyid = keys[key]
	else
		print("no such key")
		return
	end
	
	local player = 1
	if (player_ ~= nil) then
		player = player_
	end
	
	if (keyid <= 4) then
		local drs = ndirs[keyid+1]
		local ox = drs[1]
		local oy = drs[2]
		local dir = keyid
		
		movecommand(ox,oy,dir,player)
		MF_update()
	end
	
	if (keyid == 5) then
		MF_restart()
	end
	
	dolog(key)
end

function dolog(key)
	MF_log(key)
end

function createall(matdata,x_,y_,id_)
	local all = {}
	local empty = false
	
	if (x_ == nil) and (y_ == nil) and (id_ == nil) then
		if (matdata[1] ~= "empty") and (matdata[1] ~= "level") and (matdata[1] ~= "group") then
			all = findall(matdata)
		elseif (matdata[1] == "empty") then
			all = findempty()
			empty = true
		end
	end
	local test = {}
	
	if (x_ ~= nil) and (y_ ~= nil) and (id_ ~= nil) then
		local check = findtype(matdata,x_,y_,id_)
		
		if (#check > 0) then
			for i,v in ipairs(check) do
				if (v ~= 0) then
					table.insert(test, v)
				end
			end
		end
	end
	
	if (#all > 0) then
		for i,v in ipairs(all) do
			table.insert(test, v)
		end
	end
	
	if (#test > 0) then
		for i,v in ipairs(test) do
			if (empty == false) then
				local vunit = mmf.newObject(v)
				local x,y,dir = vunit.values[XPOS],vunit.values[YPOS],vunit.values[DIR],vunit.values[MOVED]
				
				for b,unit in pairs(objectlist) do
					if (b ~= "empty") and (b ~= "all") and (b ~= "level") and (b ~= "group") and (b ~= matdata[1]) then
						local protect = hasfeature(matdata[1],"is","not " .. b,v,x,y)
						
						if (protect == nil) then
							local mat = findtype({b},x,y,v)
							local tmat = findtext(x,y)
							
							if ((#mat == 0) and (#tmat == 0)) or ((#mat == 0) and (matdata[1] == "text")) then
								create(b,x,y,dir)
								
								if (matdata[1] == "text") and (#tmat > 0) then
									for c,d in ipairs(tmat) do
										local tunit = mmf.newObject(d)
										
										if (tunit.strings[UNITNAME] == "text_" .. b) then
											delete(d,0)
										end
									end
								end
							end
						end
					end
				end
			else
				local x = v % roomsize
				local y = math.floor(v / roomsize)
				local dir = 4
				
				for b,mat in pairs(objectlist) do
					if (b ~= "empty") and (b ~= "all") and (b ~= "level") and (b ~= "group") then
						local protect = hasfeature(matdata[1],"is","not " .. b,2,x,y)
						
						if (protect == nil) then
							local mat = findtype({b},x,y,0)
							local tmat = findtext(x,y)
							
							if ((#mat == 0) and (tmat == 0)) or ((#mat == 0) and (matdata[1] == "text")) then
								create(b,x,y,dir)
								
								if (matdata[1] == "text") and (tmat ~= 0) then
									delete(tmat,0)
								end
							end
						end
					end
				end
			end
		end
	end
end

function setunitmap()
	unitmap = {}
	
	for i,unit in ipairs(units) do
		local tileid = unit.values[XPOS] + unit.values[YPOS] * roomsize
		
		--print(tostring(unit.values[XPOS]) .. ", " .. tostring(unit.values[YPOS]) .. ", " .. unit.strings[UNITNAME])
		
		if (unitmap[tileid] == nil) then
			unitmap[tileid] = {}
		end

		table.insert(unitmap[tileid], unit.fixed)
	end
end

function setundo(this)
	if (this ~= nil) then
		if (this == 1) then
			updateundo = true
		elseif (this == 0) then
			updateundo = false
		end
	else
		print("undo is nil!")
		updateundo = true
	end
end

function victory()
	MF_win()
end

function poscorrect(unitid,rotation,zoom,offset)
	local unit = mmf.newObject(unitid)
	
	local midpoint = roomsize * tilesize * 0.5
	local midtile = math.floor(roomsize * 0.5) - 0.5
	
	local x,y = unit.values[XPOS],unit.values[YPOS]
	local dx = x - midtile
	local dy = y - midtile
	
	local dir = 0 - math.atan2(dy,dx) + math.rad(rotation)
	local dist = math.sqrt((dy)^2 + (dx)^2)
	
	local newx = Xoffset + midpoint + math.cos(dir) * dist * zoom * tilesize
	local newy = Yoffset + midpoint - math.sin(dir) * dist * zoom * tilesize
	
	if (unit.values[FLOAT] == 0) then
		unit.x = newx
		unit.y = newy + offset
	end
end

function stringintable(this,data)
	if (#data > 0) then
		for i,v in ipairs(data) do
			if (this == v) then
				return true
			end
		end
	end
	
	return false
end

function levelborder()
	local edgetiles = {}
	local l = map[0]
	
	for i=0,roomsize-1 do
		for j=0,roomsize-1 do
			if (i == 0) or (j == 0) or (i == roomsize-1) or (j == roomsize-1) then
				local unitid = MF_create("edge")
				local unit = mmf.newObject(unitid)
				
				table.insert(edgetiles, unitid)
				
				unit.layer = 2
				unit.values[ONLINE] = 1
				unit.values[XPOS] = i
				unit.values[YPOS] = j
				unit.x = Xoffset + i * tilesize + tilesize * 0.5
				unit.y = Yoffset + j * tilesize + tilesize * 0.5
				
				l:set(i,j,0,0)
			end
		end
	end
	
	local c1,c2 = getuicolour("edges")
	
	for i,unitid in ipairs(edgetiles) do
		local unit = mmf.newObject(unitid)
		
		local dynamicdir = dynamictile(unitid,unit.values[XPOS],unit.values[YPOS],"edge")
		
		unit.direction = dynamicdir
		
		MF_setcolour(unitid,c1,c2)
	end
end

function updatescreen(x,y)
	Xoffset = x
	Yoffset = y
end